/*
  Copyright 1999 by James Dean Palmer.
 
  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.
 
  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Please report all bugs and problems to "james@tiger-marmalade.com".  */

/*:
  @node fltk-adjuster, Top, Top
  @chapter fltk-adjuster

  @section Class Hierarchy

  @example
  fltk-widget
    |
    +---- fltk-valuator
            |
	    +---- fltk-adjuster
  @end example

  @section Description

  The fltk-adjuster class controls a single floating-point value and
  provides a consistent interface to set the value, range, and step,
  and insures that callbacks are done the same for every object.

  @section Methods

  @subsection (fltk-adjuster-new x y w h label)

  Returns a new Fl_Adjuster widget using the given position, size, and
  label string. The default boxtype is FL_NO_BOX.

*/

#include <FL/Fl_Adjuster.H>
#include "guile_fltk.h"
long fltk_adjuster_tag;

static SCM 
fltk_adjuster_new (SCM x, SCM y, SCM w, SCM h, SCM label)
{
  Fl_Adjuster * widget;
  SCM widget_smob;

  SCM_ASSERT (gh_number_p(x), x,  SCM_ARG1, "fltk-adjuster-new");
  SCM_ASSERT (gh_number_p(y), y,  SCM_ARG2, "fltk-adjuster-new");
  SCM_ASSERT (gh_number_p(w), w,  SCM_ARG3, "fltk-adjuster-new");
  SCM_ASSERT (gh_number_p(h), h,  SCM_ARG4, "fltk-adjuster-new");
  SCM_ASSERT (gh_string_p(label), label, SCM_ARG5, "fltk-adjuster-new");

  widget = new Fl_Adjuster(gh_scm2int(x), gh_scm2int(y), 
			   gh_scm2int(w), gh_scm2int(h),
			   gh_scm2newstr(label, NULL));
  
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, fltk_adjuster_tag);
  gfltk_register (widget_smob);

  return widget_smob;
}

static SCM
fltk_adjuster_get_soft(SCM widget_smob)
{
  Fl_Adjuster *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_adjuster_tag),
              widget_smob, SCM_ARG1, "fltk-adjuster-get-soft");

  widget = (Fl_Adjuster *) SCM_CDR (widget_smob);

  return gh_int2scm((uchar)widget->soft());
}


static SCM
fltk_adjuster_set_soft(SCM widget_smob, SCM a)
{
  Fl_Adjuster *widget;

  SCM_ASSERT (gfltk_type_p(widget_smob, fltk_adjuster_tag),
              widget_smob, SCM_ARG1, "fltk-adjuster-get-soft");

  SCM_ASSERT (gh_number_p(a), a,  SCM_ARG2, "fltk-adjuster-get-soft");

  uchar aa = (uchar)gh_scm2int(a);
  widget = (Fl_Adjuster *) SCM_CDR (widget_smob);
  widget->soft(aa);

  return SCM_UNSPECIFIED;
}

//***************************************************************************
//
//  The next few static procedures provide Guile with mechanisms to 
//  garbage collect and print this smob type.
//
//***************************************************************************

static SCM
widget_mark (SCM widget_smob)
{
  return SCM_BOOL_F;
}

static scm_sizet
widget_free (SCM widget_smob)
{
  delete (Fl_Adjuster *) SCM_CDR (widget_smob);
  gfltk_remove (widget_smob);
  return (sizeof (Fl_Adjuster) + sizeof (Gfltk));
}

static int
widget_print (SCM widget_smob, SCM port, scm_print_state *pstate)
{
  Fl_Adjuster *widget = (Fl_Adjuster *) SCM_CDR (widget_smob);
  scm_puts ("#<widget Fl_Adjuster ", port);
  scm_puts ((char *)widget->label(), port);
  scm_puts (">", port);
  return 1;
}

static scm_smobfuns widget_smobfuns = {
  widget_mark, 
  widget_free, 
  widget_print, 
  0
};

//***************************************************************************
//
//  Create the new fltk-adjuster scheme object type, and add the
//  associated methods as scheme procedures.
//
//***************************************************************************

void
fltk_adjuster_init ()
{
  fltk_adjuster_tag = scm_newsmob (&widget_smobfuns);

  // inherit parent's methods
  fltk_valuator_subclass("fltk-adjuster", fltk_adjuster_tag);

  // add new methods
  gh_new_procedure5_0 ("fltk-adjuster-new", 
		       fltk_adjuster_new);
  gh_new_procedure1_0 ("fltk-adjuster-get-soft", 
		       fltk_adjuster_get_soft);
  gh_new_procedure2_0 ("fltk-adjuster-set-soft", 
		       fltk_adjuster_set_soft);

}

//***************************************************************************
//
//  Mechanism to subclass this class.
//
//***************************************************************************

void
fltk_adjuster_subclass (char *s, long tag)
{
  char t[120];

  gfltk_class_register(tag, fltk_adjuster_tag);
  fltk_valuator_subclass(s, tag);

  sprintf (t, "%s-new", s);
  gh_define (t, scm_symbol_value0("fltk-adjuster-new"));
  sprintf (t, "%s-set-soft", s);
  gh_define (t, scm_symbol_value0("fltk-adjuster-set-soft"));
  sprintf (t, "%s-get-soft", s);
  gh_define (t, scm_symbol_value0("fltk-adjuster-get-soft"));

}
