// Copyright 1999 by James Dean Palmer.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.

#include <FL/Fl_Widget.H>
#include "guile_fltk.h"
#include "object_registry.h"

//***************************************************************************
//
//  The Gfltk object keeps track of a particular widget's callback and
//  associated smob.
//
//***************************************************************************

Gfltk::Gfltk(SCM a)
{
  widget_smob = a;
  widget = (Fl_Widget *) SCM_CDR (widget_smob);
  widget_callback = SCM_BOOL_F;
  scm_protect_object(widget_callback);
  scm_protect_object(widget_smob);
}

Gfltk::~Gfltk()
{
}

void 
Gfltk::callback(SCM a)
{
  widget->callback(guile_fltk_callback);
  widget->argument(a);
  //scm_unprotect_object(widget_callback);
  widget_callback = a;
  scm_protect_object(widget_callback);
}

void 
Gfltk::unprotect()
{
  scm_unprotect_object(widget_callback);
  scm_unprotect_object(widget_smob);
}

SCM 
Gfltk::callback()
{
  return widget_callback;
}

//***************************************************************************
//
//  The object_registry is a vector of all the widget smobs allocated from 
//  the Guile interface.  We can register new objects with 
//  gfltk_register(), and we can lookup a widget smob (given the widget 
//  pointer) with gfltk_lookup()
//
//***************************************************************************

static GfltkArray object_registry;

void 
gfltk_register(SCM b)
{
  Gfltk * c = new Gfltk(b);
  object_registry.append() = c;
}

SCM 
gfltk_register_as(Fl_Widget *widget, long tag)
{
  SCM widget_smob;
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, tag);
  gfltk_register(widget_smob);
  return widget_smob;
}

void 
gfltk_remove(SCM a)
{
  int i;
  for (i=0; i<object_registry.size(); i++)
    {
      if (object_registry[i]->widget_smob == a)
	{
	  delete object_registry[i];
	  object_registry.remove(i);
	  return;
	}
    }
}

SCM 
gfltk_lookup(Fl_Widget *a)
{
  int i;
  for (i=0; i<object_registry.size(); i++)
    {
      if (object_registry[i]->widget == a)
	return object_registry[i]->widget_smob;
    }
  return SCM_BOOL_F;
}

SCM 
gfltk_lookup2(Fl_Widget *a, long tag)
{
  int i;
  for (i=0; i<object_registry.size(); i++)
    {
      if (object_registry[i]->widget == a)
	return object_registry[i]->widget_smob;
    }

  return gfltk_register_as(a, tag);
}

void 
gfltk_set_callback(Fl_Widget *widget, SCM callback)
{
  int i;
  for (i=0; i<object_registry.size(); i++)
    {
      if (object_registry[i]->widget == widget)
	{
	  object_registry[i]->callback(callback);
	  return;
	}
    }
}

SCM
gfltk_get_callback(Fl_Widget *widget)
{
  int i;
  for (i=0; i<object_registry.size(); i++)
    if (object_registry[i]->widget == widget)
      return object_registry[i]->callback();
  return SCM_BOOL_F;
}

void guile_fltk_callback(Fl_Widget *a, void *b)
{
  int i=0;
  while ((i<object_registry.size()) && (object_registry[i]->widget != a))
    i++;

  if (i>=object_registry.size())
    return;

  SCM aa = object_registry[i]->callback();

  if (SCM_CLOSUREP((SCM)aa))
    gh_call2((SCM)aa, (SCM)object_registry[i]->widget_smob, (SCM)a->user_data());
}

void guile_fltk_callback2(Fl_Widget *a, void *b)
{
  SCM aa = (SCM)b;

  if (SCM_CLOSUREP((SCM)aa))
    gh_call0((SCM)aa);
}

//***************************************************************************
//
//  The class_registry is a vector that preserves c++ inheritance 
//  relationships.  A class is registered by passing the new smob type
//  and the parent smob type to gfltk_class_register(child_type, 
//  parent_type).  To check if a widget is a child of a parent class, you 
//  can then use gfltk_class_check(child, parent_type)
//
//***************************************************************************

struct class_entry
{
  long a;
  long b;
};

#include "class_registry.h"

static ClassArray class_registry;

void gfltk_class_register(long a, long b)
{
  class_entry * m = new class_entry;
  m->a = a;
  m->b = b;
  class_registry.append() = m;
}

int gfltk_type_p(SCM a, long b)
{
  int i;
  long aa = gh_car (a);
  if (!SCM_NIMP (a))
    return 0;

  if (aa == b) return 1;
  for (i=0; i<class_registry.size(); i++)
    {
      if (class_registry[i]->a == aa)
	if (class_registry[i]->b == b)
	  return 1;
    }
  return 0;
}

//***************************************************************************
//
//  gfltk_export_widget is a function specifically designed to export
//  c++ initiated widgets to the Guile object name space.
//
//***************************************************************************

void
gfltk_export_object (char *name, Fl_Widget *widget, long widget_tag)
{
  SCM widget_smob;
  SCM_NEWCELL (widget_smob);
  SCM_SETCDR (widget_smob, widget);
  SCM_SETCAR (widget_smob, widget_tag);
  gfltk_register (widget_smob);
  gh_define (name, widget_smob);
}

//***************************************************************************
//
//  When the fltk module is initially loaded we need to register each
//  fltk data type and define the fltk scheme procedures.
//
//***************************************************************************

void
scm_init_gfltk()
{
  int verbose = 0;

  gh_eval_str("(use-modules (ice-9 optargs))");

  if (verbose)
    printf("Initializing Fl..\n");
  fltk_fl_init();
  if (verbose)
    printf("Initializing Widget..\n");
  fltk_widget_init();
  if (verbose)
    printf("Initializing Group..\n");
  fltk_group_init();
  if (verbose)
    printf("Initializing Tile..\n");
  fltk_tile_init();
  if (verbose)
    printf("Initializing Menu..\n");
  fltk_menu_init();
  if (verbose)
    printf("Initializing Menu_Bar..\n");
  fltk_menu_bar_init();
  if (verbose)
    printf("Initializing Browser..\n");
  fltk_browser_base_init();
  if (verbose)
    printf("Initializing Browser Base..\n");
  fltk_browser_init();
  if (verbose)  
    printf("Initializing Hold Browser..\n");
  fltk_hold_browser_init();
  if (verbose)  
    printf("Initializing Multi Browser..\n");
  fltk_multi_browser_init();
  if (verbose)  
    printf("Initializing Select Browser..\n");
  fltk_select_browser_init();
  if (verbose)  
    printf("Initializing Window..\n");
  fltk_window_init();
  if (verbose)  
    printf("Initializing Button..\n");
  fltk_button_init();
  if (verbose)  
    printf("Initializing Box..\n");
  fltk_box_init();
  if (verbose)  
    printf("Initializing Check Button..\n");
  fltk_check_button_init();
  if (verbose)  
    printf("Initializing Light Button..\n");
  fltk_light_button_init();
  if (verbose)  
    printf("Initializing Repeat Button..\n");
  fltk_repeat_button_init();
  if (verbose)  
    printf("Initializing Return Button..\n");
  fltk_return_button_init();
  if (verbose)  
    printf("Initializing Round Button..\n");
  fltk_round_button_init();
  if (verbose)  
    printf("Initializing Valuator..\n");
  fltk_valuator_init();
  if (verbose)  
    printf("Initializing Adjuster..\n");
  fltk_adjuster_init();
  if (verbose)  
    printf("Initializing Counter..\n");
  fltk_counter_init();
  if (verbose)  
    printf("Initializing Dial..\n");
  fltk_dial_init();
  if (verbose)  
    printf("Initializing Roller..\n");
  fltk_roller_init();
  if (verbose)  
    printf("Initializing Slider..\n");
  fltk_slider_init();
  if (verbose)  
    printf("Initializing Value Input..\n");
  fltk_value_input_init();
  if (verbose)  
    printf("Initializing Value Output..\n");
  fltk_value_output_init();
  if (verbose)  
    printf("Initializing Input_ (base)..\n");
  fltk_input_base_init();
  if (verbose)  
    printf("Initializing Input..\n");
  fltk_input_init();
  if (verbose)  
    printf("Initializing Float Input..\n");
  fltk_float_input_init();
  if (verbose)  
    printf("Initializing Int Input..\n");
  fltk_int_input_init();
  if (verbose)  
    printf("Initializing Secret Input..\n");
  fltk_secret_input_init();
  if (verbose)  
    printf("Initializing Multiline Input..\n");
  fltk_multiline_input_init();
  if (verbose)  
    printf("Initializing Chart..\n");
  fltk_chart_init();
  if (verbose)  
    printf("Initializing Choice..\n");
  fltk_choice_init();
  if (verbose)  
    printf("Initializing Color Chooser..\n");
  fltk_color_chooser_init();
  if (verbose)  
    printf("Initializing Clock..\n");
  fltk_clock_init();
  if (verbose)  
    printf("Initializing Round Clock..\n");
  fltk_round_clock_init();
  if (verbose)  
    printf("Initializing Menu Button..\n");
  fltk_menu_button_init();

  if (verbose)  
    printf("Completed Fltk Init..\n");
  scm_gc();
}

//***************************************************************************
//
//  This provides the "module hook" into the shared library.
//
//***************************************************************************

void
scm_init_gfltk_module()
{
  scm_register_module_xxx("gfltk", (void*)scm_init_gfltk);
}

/* 
   A Sample program:

   (use-modules (gfltk))
   (define mywindow (fltk-window-new 30 30 200 200 "Hello World Window"))

   (define mywindow (fltk-window-new "Hello World Window" #:w 400 #:h 600))
   (define mybutton (fltk-button-new 10 30 100 50 "Hello World"))
   (define mymenu (fltk-menu-bar-new 0 0 200 24 "Blah"))
   (fltk-window-add mywindow mybutton)
   (fltk-window-add mywindow mymenu)1g
   (fltk-window-show mywindow)
   (fltk-button-show mybutton)
   (fltk-menu-bar-show mymenu)
   (define quitcallback (lambda () (display "I am quiting!!\n") 
   (fltk-window-hide mywindow) (quit)))
   (define opencallback (lambda () (display "I am opening a file!!\n")))
   (define closecallback (lambda () (display "I am closing a file!!\n")))
   (fltk-button-set-callback mybutton quitcallback)
   (fltk-menu-bar-add mymenu "File" "" 0 FL_SUBMENU)
   (fltk-menu-bar-add mymenu "File/Open" "" opencallback 0)
   (fltk-menu-bar-add mymenu "File/Close" "" closecallback 0)
   (fltk-menu-bar-add mymenu "File/Quit" "" quitcallback 0)
   (fltk-run)

*/

